class Course(object):
    
    # These scales come from U of T's Faculty of Arts and Sciences.
    
    GRADE_TO_GPA = {
        85: 4.0,
        80: 3.7,
        77: 3.3,
        73: 3.0,
        70: 2.7,
        67: 2.3,
        63: 2.0,
        60: 1.7,
        57: 1.3,
        53: 1.0,
        50: 0.7,
        0: 0.0
    }
    
    GRADE_TO_LETTER = {
        90: 'A+',
        85: 'A',
        80: 'A-',
        77: 'B+',
        73: 'B',
        70: 'B-',
        67: 'C+',
        63: 'C',
        60: 'C-',
        57: 'D+',
        53: 'D',
        50: 'D-',
        0: 'F'
        }
    
    
    GRADE_TO_GPA_ORDERED = list(GRADE_TO_GPA.keys())
    GRADE_TO_GPA_ORDERED.sort(reverse=True)
    
    GRADE_TO_LETTER_ORDERED = list(GRADE_TO_LETTER.keys())
    GRADE_TO_LETTER_ORDERED.sort(reverse=True)    
    
    def __init__(self, name):
        '''(str, float) -> NoneType
        
        Create a Course object with name, a bonus of 0, and an empty list of
        assignments.
        '''
        
        self._name = name
        self._assns = []
        self._bonus = 0
        
        
    def __str__(self):
        return self._name
    
    
    def rename(self, name):
            self._name = name
            
    
    def get_contents(self):
        return self._assns
    
    
    def get_bonus(self):
        return self._bonus
    
    
    def set_bonus(self, bonus):
        '''(float) -> NoneType
        
        Add a bonus or penalty mark to be applied to all future grades.
        '''
        
        self._bonus = bonus
    
        
    def add_assn(self, assn):
        '''(Assignment) -> NoneType
        
        Add the assignment to the course.
        '''
        
        self._assns.append(assn)
            
        
    def del_assn(self, assn):
        '''(Assignment) -> NoneType
        
        Remove the assignment from the course.
        '''
        
        self._assns.remove(assn)
        
        
    def calculate_GPA_and_letter(self, grade):
        '''(float) -> float or int
        
        Take the grade and return a GPA, or -1 if there is no grade yet.
        '''
               
        if grade != -1:
            for key in self.GRADE_TO_GPA_ORDERED:
                if grade >= key:
                    GPA = self.GRADE_TO_GPA[key]
                    break
                
            for key in self.GRADE_TO_LETTER_ORDERED:
                if grade >= key:
                    letter = self.GRADE_TO_LETTER[key]  
                    break
                
            return GPA, letter
        
        else:
            return -1, ''
    
    
    def get_free_weight(self):
        '''() -> float
        
        Return how much weight can still be allotted to assignments in this
        course.
        '''
        
        free = 100.0
        for assn in self._assns:
            free -= assn.get_weight()
            
        return free
    
        
    def get_grade(self):
        '''() -> tuple of float or int, float, float, float or int
        
        Add up the known weight and grade from the assignments, plus bonus.
        Return a tuple of the weighted grade, the absolute percent received on
        all graded assignments, the absolute percent possible on all graded
        assignments, the GPA, and the letter.
        
        '''
        
        known = 0.0
        received = 0.0
        
        for assn in self._assns:
            if assn.get_grade() != -1:
                known += assn.get_weight()
                received += assn.get_weight() * assn.get_grade()
        
        try:
            grade = 100 * received / known
            grade = grade + self._bonus
        except(ZeroDivisionError):
                grade = -1
            
        GPA, letter = self.calculate_GPA_and_letter(grade) 
        return grade, received, known, GPA, letter